<?php

if (substr(php_sapi_name(), 0, 3) != 'cli') exit;

include "bootstrap.php";

$app = new \Espo\Core\Application();
$app->setupSystemUser();

$mb4Converter = new InnoDbConverter($app->getContainer());
$mb4Converter->run();

class InnoDbConverter
{
    private $container;

    private $databaseHelper;

    protected $dbRequirements = [
        'mysql' => '5.6.4',
        'mariadb' => '10.0.5',
    ];

    public function __construct($container)
    {
        $this->container = $container;
        $this->databaseHelper = new \Espo\Core\Utils\Database\Helper($this->container->get('config'));
    }

    protected function getContainer()
    {
        return $this->container;
    }

    protected function getDatabaseHelper()
    {
        return $this->databaseHelper;
    }

    public function run()
    {
        fwrite(\STDOUT, "We strongly recommend to create a backup of your database, https://docs.espocrm.com/administration/backup-and-restore/#step-2-backup-database.\n");
        sleep(1);

        fwrite(\STDOUT, "All tables with MyISAM storage engine will be converted to InnoDB. Enter [Y] to continue.\n");

        if (!$this->confirm()) {
            echo "Canceled.\n";
            return;
        }

        if ($this->displayFullTextNotice()) {
            $dbType = $this->getDatabaseHelper()->getDatabaseType();
            $dbTypeLc = strtolower($dbType);

            fwrite(\STDOUT, "Please note that your InnoDB engine does not support full-text indexes. We recommend upgrading your ". $dbType ." to at least version ". $this->dbRequirements[$dbTypeLc] ." and then start the process again. If you insist on continuing, enter [Y].\n");

            if (!$this->confirm()) {
                echo "Canceled.\n";
                return;
            }
        }

        $this->convert();
    }

    protected function confirm()
    {
        $fh = fopen('php://stdin', 'r');
        $inputLine = trim(fgets($fh));
        fclose($fh);
        if (strtolower($inputLine) !== 'y'){
            return false;
        }
        return true;
    }

    protected function convert()
    {
        $container = $this->getContainer();

        $pdo = $container->get('entityManager')->getPDO();
        $ormMeta = $container->get('ormMetadata')->getData(true);

        $databaseInfo = $container->get('config')->get('database');

        try {
            $sth = $pdo->prepare("SELECT TABLE_NAME as tableName FROM information_schema.TABLES WHERE TABLE_SCHEMA = '". $databaseInfo['dbname'] ."' and ENGINE = 'MyISAM'");
            $sth->execute();
        } catch (\Exception $e) {
            fwrite(\STDOUT, "Error: Cannot get table list.\n");
            exit;
        }

        $tableList = $sth->fetchAll(\PDO::FETCH_NUM);

        if (empty($tableList)) {
            fwrite(\STDOUT, "All is well. No MyISAM tables found.\n");
            exit;
        }

        foreach ($tableList as $item) {

            $tableName = $item[0];

            fwrite(\STDOUT, "Table: " . $tableName);

            $query = "ALTER TABLE `". $tableName ."` ENGINE=InnoDB;";

            try {
                $sth = $pdo->prepare($query);
                $sth->execute();
            } catch (\Exception $e) {
                $GLOBALS['log']->debug('InnoDbConverter: FAILED convering "'. $tableName .'" table to InnoDB, details: ' . $e->getMessage());
                continue;
            }

            fwrite(\STDOUT, "\n");
        }

        $this->getContainer()->get('dataManager')->rebuild();

        fwrite(\STDOUT, "Successfully completed.\n");
    }

    protected function getColumnType($type)
    {
        if (preg_match('/^(.*)\(/i', $type, $match)) {
            return strtolower($match[1]);
        }

        return strtolower($type);
    }

    protected function displayFullTextNotice()
    {
        $dbType = $this->getDatabaseHelper()->getDatabaseType();
        $dbType = strtolower($dbType);
        $dbVersion = $this->getDatabaseHelper()->getDatabaseVersion();

        if (isset($this->dbRequirements[$dbType]) && version_compare($dbVersion, $this->dbRequirements[$dbType], '<')) {
            return true;
        }

        return false;
    }
}